#include <iostream>
#include <fstream>
#include <string>

#include <assert.h>

#include "mea_cmdline.h"

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif


#include "LocARNA/aux.hh"
#include "LocARNA/confusion_matrix.hh"
#include "LocARNA/rna_structure.hh"

#include "mea_predictor.hh"

//typedef MeaSpanPenaltyPredictor<MeaCLPredictor> predictor_t;
typedef MeaSpanPenaltyPredictor<MeaOSPredictor> predictor_t;

int
main(int argc, char *argv[])
{
  gengetopt_args_info args_info;
  
  if (cmdline_parser(argc,argv,&args_info) != 0) {
    return -1;
  }
  
  if(args_info.plot_penalty_given)
    {
      predictor_t 
	mp("",LocARNA::SparseMatrix<double>(0), 
	   args_info.alpha_arg,
	   args_info.beta_arg, 
	   args_info.gamma_arg,
	   args_info.delta_arg);

      mp.print_penalty(args_info.plot_penalty_arg);
      cmdline_parser_free(&args_info);
      return 0;
    }
  
  if (args_info.inputs_num!=1) {
    cmdline_parser_print_help();
    cmdline_parser_free(&args_info);
    exit(-1);
  }
  
  std::string filename = args_info.inputs[0];
  
  try {
    std::ifstream in(filename.c_str());
      
      if (! in.good()) {
	  throw std::ios_base::failure("Cannot open "+filename+" for reading.");
      }

      predictor_t 
	mp(in, 
	   args_info.alpha_arg,
	   args_info.beta_arg, 
	   args_info.gamma_arg,
	   args_info.delta_arg);
      
      in.close();


      double score = mp.fold(); // dp
      //traceback
      std::string predicted_structure = mp.traceback();
      
      std::cout << mp.sequence() << std::endl;
      std::cout << predicted_structure << " (" << score <<")" << std::endl;
      
      if (args_info.reference_given) {
	  const std::string reference_structure = args_info.reference_arg;
	  if (reference_structure.length() != mp.sequence().length()) {
	      std::cerr << "Failure: length of sequence and refernence differ."
			<< std::endl;
	      return -1;      
	  }
	  
	  try {
	    LocARNA::RnaStructure refstruct(reference_structure);
	    LocARNA::RnaStructure predstruct(predicted_structure);
	    LocARNA::ConfusionMatrix cm(refstruct,predstruct);
	      
	      // write TP FP FN TN SENS PPV F1 MCC
	      std::cout << cm.tp() << " "
			<< cm.fp() << " "
			<< cm.fn() << " "
			<< cm.tn() << " "
			<< cm.sens() << " "
			<< cm.ppv() << " "
			<< cm.f1_score() << " "
			<< cm.mcc() << std::endl;
	      
	  } catch (LocARNA::failure f) {
	      std::cerr << "Failure: "<<f.what()<<std::endl;
	      cmdline_parser_free(&args_info);
	      return -1;
	  }
	  
      }
      
  } catch (std::ios_base::failure f) {
      std::cerr <<f.what()<<std::endl; 
      cmdline_parser_free(&args_info);
      return -1;
  }
  
  cmdline_parser_free(&args_info);
  return 0;
}
